package ui.model;

import Interfaces.CategoryListener;
import Interfaces.ObjectListener;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

import classes.Category;
import classes.CpsEdge;
import classes.CpsObject;
import ui.controller.*;

public class Model {

	//Global Variables
	private static int SCALE = 50;	//Picture Scale
	private static int SCALE_DIV2 = SCALE/2;
	private static final int ITERATIONS = 50;
	
	// ID of the Selected Object
	private int selectedID = 0;
	// eventuell wenn Canvasgröße gewählt werden kann
	private int HEIGHT;
	private int WIDTH;
	/*
	 * Array of all categories in the model. It is set by default with the
	 * categories ENERGY, BUILDINGS and COMPONENTS
	 */
	private ArrayList<Category> categories;
	/*
	 * Array of all CpsObjects in our canvas. It is set by default as an empty
	 * list.
	 */
	private ArrayList<CpsObject> objectsOnCanvas;
	/*
	 * Array of all CpsObjects in our canvas. It is set by default as an empty
	 * list.
	 */
	private ArrayList<CpsEdge> edgesOnCanvas;

	/*
	 * Array for all Listeners
	 */
	private List<CategoryListener> categoryListeners;
	private List<ObjectListener> objectListeners;
	

	/*
	 * Constructor for the model. It initializes the categories and
	 * objectsOnCanvas by default values. Listeners are also initialized by
	 * default values.
	 */
	public Model() {
		setCategories(new ArrayList<Category>());
		setObjectsOnCanvas(new ArrayList<CpsObject>());
		setEdgesOnCanvas(new ArrayList<CpsEdge>());
		setCategoryListeners(new LinkedList<CategoryListener>());
		setObjectListeners(new LinkedList<ObjectListener>());
	}

	/**
	 * @return the categories
	 */
	public ArrayList<Category> getCategories() {
		return categories;
	}

	/**
	 * @param categories
	 *            the categories to set
	 */
	public void setCategories(ArrayList<Category> categories) {
		this.categories = categories;
	}

	/**
	 * Transform the Arraylist of categories into a string of all objectName
	 * with a separation (',') between each name
	 * 
	 * @return String of all names separeted by ','
	 */
	public String toStringCat() {
		String text = "";
		for (int i = 0; i < categories.size(); i++) {
			if (text == "") {
				text = categories.get(i).getName();
			} else {
				text = text + ", " + categories.get(i).getName();
			}
		}
		return text;
	}

	/**
	 * @return the objectsOnCanvas
	 */
	public ArrayList<CpsObject> getObjectsOnCanvas() {
		return objectsOnCanvas;
	}

	/**
	 * @param objectsOnCanvas
	 *            the objectsOnCanvas to set
	 */
	public void setObjectsOnCanvas(ArrayList<CpsObject> objectsOnCanvas) {
		this.objectsOnCanvas = objectsOnCanvas;
	}
	
	/**
	 * @return the objectsOnCanvas
	 */
	public ArrayList<CpsEdge> getEdgesOnCanvas() {
		return edgesOnCanvas;
	}

	/**
	 * @param objectsOnCanvas
	 *            the objectsOnCanvas to set
	 */
	public void addEdgeOnCanvas(CpsEdge edge) {
		this.edgesOnCanvas.add(edge);
	}
	
	/**
	 * @param edgesOnCanvas
	 *            the edge to remove
	 */
	public void removeEdgesOnCanvas(CpsEdge edge) {
		this.edgesOnCanvas.remove(edge);
	}
	
	/**
	 * @param EdgesOnCanvas
	 *            the edgesOnCanvas to set
	 */
	public void setEdgesOnCanvas(ArrayList<CpsEdge> arrayList) {
		this.edgesOnCanvas = arrayList;
	}
	
	/**
	 * @return the objectListeners
	 */
	public List<ObjectListener> getObjectListeners() {
		return objectListeners;
	}

	/**
	 * @param linkedList
	 *            the objectListeners to set
	 */
	public void setObjectListeners(LinkedList<ObjectListener> linkedList) {
		this.objectListeners = linkedList;
	}

	/**
	 * @return the categoryListeners
	 */
	public List<CategoryListener> getCategoryListeners() {
		return categoryListeners;
	}

	/**
	 * @param linkedList
	 *            the categoryListeners to set
	 */
	public void setCategoryListeners(LinkedList<CategoryListener> linkedList) {
		this.categoryListeners = linkedList;
	}
	
	/**
	 * Set the ID of the selected Object
	 * 0 = no Object is selected
	 * 
	 * @param ID
	 * 
	 */
	public void setSelectedObjectID(int id){
		this.selectedID = id;
	}
	
	/**
	 * Returns the ID of the selected Object
	 * 0 = no Object is selected
	 * 
	 * @return ID
	 */
	public int getSelectedObjectID(){
		return selectedID;
	}
	
	/**
	 * Returns SCALE
	 * 
	 * @return SCALE
	 */
	public int getScale(){
		return SCALE;
	}
	
	/**
	 * Returns SCALE_DIV2
	 * 
	 * @return SCALE_DIV2
	 */
	public int getScaleDiv2(){
		return SCALE_DIV2;
	}
	
	public void setScale(int scale){
		SCALE = scale;
		SCALE_DIV2 = SCALE/2;
	}
	
	/**
	 * Returns ITERATIONS
	 * 
	 * @return ITERATIONS
	 */
	public int getIterations(){
		return ITERATIONS;
	}

}
